#######################################################
##
## Store-Config-In-Svn
##
## Copyright 2012 Zany Ants Ltd. All rights reserved.
##
#######################################################

<#

.SYNOPSIS

Retrieves the configurations from a number of servers using Xoom,
applies the defined transformations to the configurations, and
stores the result in subversion folders specified.

.DESCRIPTION

The configuration for the script is contained in the file called
Store-Config-In-Svn.xml, which defines, for each server, what
query to run, what transformations to apply after the configuration
has been retrieved and where to store the resulting Xoom file.
The configuration file is self-documenting.

#>

function CreateAndAddToSvn-Folder( $folder )
{
  if ( Test-Path $folder ) { return }
  
  $parent = Split-Path $folder
  $name = Split-Path $folder -leaf
  CreateAndAddToSvn-Folder( $parent )
  New-Item -name $name -type directory -path $parent
  svn add $folder
}

Push-Location
$curPath = Split-Path -parent $MyInvocation.MyCommand.Definition
cd $curPath

$xml = [xml] ( Get-Content Store-Config-In-Svn.xml )

# Checkout the working copies for all the repos that haven't been checked out yet
foreach ( $repo in $xml.Config.SvnRepos.SvnRepo )
{
  if ( -not ( Test-Path $repo.WorkingCopy ) )
  {
    svn checkout $repo.Uri $repo.WorkingCopy
  }
}

# Process all Xoom queries
foreach ( $query in $xml.Config.XoomQueries.XoomQuery )
{
  # Create the directory structure and the file, if necessary
  if ( -not ( Test-Path $query.SvnPath ) )
  {
    $parent = Split-Path $query.SvnPath
    CreateAndAddToSvn-Folder( $parent )
    New-Item -name ( Split-Path $query.SvnPath -leaf ) -type file -path $parent
    svn add $query.SvnPath
  }

  # Build and execute the command
  $getQuery = $query.GetQuery
  $hostName = $query.HostName
  $command = ".\XoomToolkit send --server=$hostName"
  if ( $query.Transform -ne $null ) { $command = $command + ' --transform=' + $query.Transform }
  $command = $command + " ""<Get><Query>$getQuery<Get><Query>"" "
  $res = [xml] ( Invoke-Expression $command )
  $res.Save( $query.SvnPath )
}

# Commit the changes
foreach ( $repo in $xml.Config.SvnRepos.SvnRepo )
{
  svn commit $repo.WorkingCopy -m ( ( Get-Date ).ToShortDateString() + ': Xoom versioning' )
}

Pop-Location
